Package: CellTrails
Type: Package
Title: Reconstruction, visualization and analysis of branching
        trajectories
Version: 1.29.0
Authors@R: person(given="Daniel", family="Ellwanger", 
  role=c("aut", "cre", "cph"), email="dc.ellwanger.dev@gmail.com")
Description: CellTrails is an unsupervised algorithm for the de novo 
  chronological ordering, visualization and analysis of single-cell expression 
  data. CellTrails makes use of a geometrically motivated concept of 
  lower-dimensional manifold learning, which exhibits a multitude of virtues that 
  counteract intrinsic noise of single cell data caused by drop-outs, technical 
  variance, and redundancy of predictive variables. CellTrails enables the 
  reconstruction of branching trajectories and provides an intuitive graphical 
  representation of expression patterns along all branches simultaneously. It 
  allows the user to define and infer the expression dynamics of individual and 
  multiple pathways towards distinct phenotypes.
License: Artistic-2.0
Encoding: UTF-8
VignetteBuilder: knitr
Depends: R (>= 3.5), SingleCellExperiment
Imports: BiocGenerics, Biobase, cba, dendextend, dtw, EnvStats,
        ggplot2, ggrepel, grDevices, igraph, maptree, methods, mgcv,
        reshape2, Rtsne, stats, splines, SummarizedExperiment, utils
Suggests: AnnotationDbi, destiny, RUnit, scater, scran, knitr,
        org.Mm.eg.db, rmarkdown
RoxygenNote: 7.1.0
biocViews: ImmunoOncology, Clustering, DataRepresentation,
        DifferentialExpression, DimensionReduction, GeneExpression,
        Sequencing, SingleCell, Software, TimeCourse
Collate: 'AllClasses.R' 'AllGenerics.R' 'accessor-methods.R'
        'cluster-methods.R' 'data-sets.R' 'diffexp-methods.R'
        'dimred-methods.R' 'export-methods.R' 'filter-methods.R'
        'fitting-methods.R' 'import-methods.R' 'imputation-methods.R'
        'layout-methods.R' 'plot-methods.R' 'show-methods.R'
        'simulation-methods.R' 'test-methods.R' 'trajectory-methods.R'
        'utility-methods.R'
git_url: https://git.bioconductor.org/packages/CellTrails
git_branch: devel
git_last_commit: 75d815a
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-03
NeedsCompilation: no
Packaged: 2025-11-03 21:41:03 UTC; biocbuild
Author: Daniel Ellwanger [aut, cre, cph]
Maintainer: Daniel Ellwanger <dc.ellwanger.dev@gmail.com>
