# Generated by roxygen2: do not edit by hand

export("latentSpace<-")
export("manifold2D<-")
export("stateTrajLayout<-")
export("states<-")
export("trailNames<-")
export("trajFeatureNames<-")
export("trajLayout<-")
export("userLandmarks<-")
export(addTrail)
export(connectStates)
export(contrastTrailExpr)
export(embedSamples)
export(enrichment.test)
export(filterTrajFeaturesByCOV)
export(filterTrajFeaturesByDL)
export(filterTrajFeaturesByFF)
export(findSpectrum)
export(findStates)
export(fitDynamic)
export(fitTrajectory)
export(landmarks)
export(latentSpace)
export(manifold2D)
export(pca)
export(phenoNames)
export(plotDynamic)
export(plotManifold)
export(plotMap)
export(plotStateExpression)
export(plotStateSize)
export(plotStateTrajectory)
export(plotTrail)
export(plotTrajectoryFit)
export(read.ygraphml)
export(removeTrail)
export(selectTrajectory)
export(showTrajInfo)
export(simulate_exprs)
export(states)
export(trailNames)
export(trails)
export(trajComponents)
export(trajFeatureNames)
export(trajLayout)
export(trajResiduals)
export(trajSampleNames)
export(userLandmarks)
export(write.ygraphml)
exportMethods(featureNames)
exportMethods(sampleNames)
import(SingleCellExperiment)
import(ggplot2)
import(methods)
importFrom(Biobase,featureNames)
importFrom(Biobase,sampleNames)
importFrom(BiocGenerics,as.data.frame)
importFrom(EnvStats,enormCensored)
importFrom(EnvStats,twoSampleLinearRankTestCensored)
importFrom(Rtsne,Rtsne)
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(cba,order.optimal)
importFrom(dendextend,rotate)
importFrom(dtw,asymmetric)
importFrom(dtw,dtw)
importFrom(ggrepel,geom_label_repel)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,hcl)
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,add.edges)
importFrom(igraph,components)
importFrom(igraph,degree)
importFrom(igraph,delete.vertices)
importFrom(igraph,distances)
importFrom(igraph,ecount)
importFrom(igraph,ends)
importFrom(igraph,get.edgelist)
importFrom(igraph,get.shortest.paths)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,induced_subgraph)
importFrom(igraph,layout_with_fr)
importFrom(igraph,mst)
importFrom(igraph,neighbors)
importFrom(igraph,vcount)
importFrom(maptree,clip.clust)
importFrom(maptree,draw.clust)
importFrom(mgcv,gam)
importFrom(reshape2,melt)
importFrom(splines,bs)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,as.dist)
importFrom(stats,coefficients)
importFrom(stats,cophenetic)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,ecdf)
importFrom(stats,fisher.test)
importFrom(stats,formula)
importFrom(stats,integrate)
importFrom(stats,lm)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.fail)
importFrom(stats,p.adjust)
importFrom(stats,phyper)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,rgamma)
importFrom(stats,rnbinom)
importFrom(stats,sd)
importFrom(stats,smooth.spline)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(utils,combn)
importFrom(utils,head)
