#' @import methods
# #' @rawNamespace import(stats, except=weights)
#' @importFrom stats aggregate anova as.dist coefficients lm mad
#' @importFrom stats cophenetic cor dist ecdf fisher.test formula
#' @importFrom stats integrate median na.fail p.adjust phyper predict
#' @importFrom stats qnorm quantile residuals rgamma rnbinom sd
#' @importFrom stats smooth.spline var wilcox.test
NULL
